

def reorder_fields(fields, start_fields, end_fields):
    field_map = {f['name']: f for f in fields}

    start = [field_map[name] for name in start_fields if name in field_map]

    end = [field_map[name] for name in end_fields if name in field_map]

    middle = [
        f for f in fields
        if f['name'] not in start_fields and f['name'] not in end_fields
    ]

    return [
        *start,
        *middle,
        *end
    ]


from rest_framework import serializers

from django.contrib.auth.models import Group
from django_resaas.core.base.serializers import BaseSerializer

class GrupoSerializer(BaseSerializer):
    class Meta:
        model = Group
        fields = "__all__"



from django.db import models
from django_resaas.core.base.models import TimeModel


class ModeloExtra(TimeModel):
    modelo = models.CharField(max_length=100, null=True)
    icon = models.CharField(max_length=100, null=True)
    method = models.CharField(max_length=50, null=True)
    permission = models.CharField(max_length=100, null=True)
    details = models.BooleanField(default=False)
    url = models.CharField(max_length=300, null=True)

    class Meta:
        permissions = ()

    def __str__(self):
        return self.modelo


# class ModeloExtra(TimeModel):
#     modelo = models.CharField(max_length=100, null=True)

#     method = models.CharField(max_length=50, null=True)
#     url = models.CharField(max_length=300, null=True)

#     # 🔥 UI
#     details = models.CharField(max_length=200, null=True, blank=True)
#     icon = models.CharField(max_length=100, null=True, blank=True)
#     color = models.CharField(max_length=50, null=True, blank=True)

#     # 🔐 segurança
#     permission = models.CharField(max_length=100, null=True, blank=True)

#     # ⚠️ UX
#     confirm = models.BooleanField(default=False)
#     confirm_message = models.CharField(max_length=200, null=True, blank=True)

#     success_message = models.CharField(max_length=200, null=True, blank=True)
#     error_message = models.CharField(max_length=200, null=True, blank=True)

#     class Meta:
#         permissions = ()






from rest_framework import serializers


from django_resaas.models.documento import Documento
from django_resaas.core.base.serializers import BaseSerializer

class DocumentoSerializer(BaseSerializer):
    class Meta:
        model = Documento
        fields = "__all__"



from django.urls import path
from rest_framework.routers import DefaultRouter
from rest_framework.viewsets import ViewSetMixin
from django_resaas.core.base.registry import VIEW_REGISTRY


def build_saas_urls():
    router = DefaultRouter()
    extra_patterns = []

    for module, views in VIEW_REGISTRY.items():

        for name, view_cls in views.items():
            prefix = f"{module}/{name}"

            if issubclass(view_cls, ViewSetMixin):
                router.register(prefix, view_cls, basename=prefix.replace("/", "_"))
            else:
                extra_patterns.append(
                    path(f"{prefix}/", view_cls.as_view())
                )

    return router, extra_patterns



from django_resaas.core.base.serializers import BaseSerializer
from django_resaas.models.ficheiro import Ficheiro


class FicheiroGravarSerializer(BaseSerializer):
    class Meta:
        model = Ficheiro
        fields = "__all__"



from rest_framework import serializers

from django.contrib.contenttypes.models import ContentType
from django_resaas.core.base.serializers import BaseSerializer

class ModeloSerializer(BaseSerializer):
    class Meta:
        model = ContentType
        fields = "__all__"



def build_select_data(queryset):
    data = []

    for obj in queryset:
        if hasattr(obj, "get_label") and hasattr(obj, "get_value"):
            data.append({
                "label": obj.get_label(),
                "value": obj.get_value()
            })
        else:
            data.append({
                "label": str(obj),
                "value": getattr(obj, "id", None)
            })

    return data


from django.conf import settings
from django.template import loader
from django.template.exceptions import TemplateDoesNotExist


DEFAULT_TEMPLATES = {
    'REGISTER_CONFIRM': 'django_resaas/email_confirmacao.html',
    'PASSWORD_RESET': 'django_resaas/email_reset.html',
    'GENERIC_RESET': 'django_resaas/email_template_reset.html',
}


def render_email_template(key, context):
    """
    Resolve template via settings com fallback para template da lib.
    """
    custom_templates = getattr(settings, 'DJANGO_REST_AUTH', {}).get('EMAIL_TEMPLATES', {})

    template_name = custom_templates.get(key, DEFAULT_TEMPLATES.get(key))

    try:
        return loader.render_to_string(template_name, context)
    except TemplateDoesNotExist:
        # fallback final de segurança
        return loader.render_to_string(DEFAULT_TEMPLATES[key], context)



from rest_framework import serializers

from django_resaas.models.idioma import Idioma
from django_resaas.core.base.serializers import BaseSerializer


class IdiomaSerializer(BaseSerializer):
    class Meta:
        model = Idioma
        fields = "__all__"



# Generated by Django 5.2.10 on 2026-03-08 13:52

import django.db.models.deletion
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_resaas', '0004_entidade_site'),
    ]

    operations = [
        migrations.CreateModel(
            name='CorsAllowedOrigin',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('deleted_at', models.DateTimeField(blank=True, null=True)),
                ('estado', models.IntegerField(choices=[(0, 'Inativo'), (1, 'Ativo')], default=0, null=True)),
                ('origin', models.URLField(unique=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(class)s_created', to=settings.AUTH_USER_MODEL)),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='%(class)s_updated', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# =========================
# Python standard library
# =========================
import base64
import os
import random


# =========================
# Third-party
# =========================
import barcode
import qrcode
from barcode.writer import ImageWriter
from PIL import Image


# =========================
# Django
# =========================
from django.conf import settings


# =========================
# Django REST Framework
# =========================
from rest_framework import filters
from rest_framework import status
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response


# =========================
# Local application (absolute imports)
# =========================
from django_resaas.core.utils.translate import Translate
from django_resaas.core.services.disc_manager import DiskManegarService

from django_resaas.models.entidade import Entidade
from django_resaas.models.ficheiro import Ficheiro
from django_resaas.models.sucursal import Sucursal
from django_resaas.models.sucursal_user_group import SucursalUserGroup

from django_resaas.data.sucursal.serializers.sucursal import SucursalSerializer
from django_resaas.data.entidade.serializers.entidade import EntidadeSerializer
from django_resaas.data.ficheiro.serializers.ficheiro import FicheiroSerializer
from django_resaas.data.ficheiro.serializers.ficheiro_gravar import FicheiroGravarSerializer






class  SucursalAPIView(viewsets.ModelViewSet):
    #permission_classes = (permissions.IsAuthenticated)
    search_fields = ['id','nome']
    filter_backends = (filters.SearchFilter,)
    
    serializer_class = SucursalSerializer
    queryset = Sucursal.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        return self.queryset.filter().order_by('-id')

    @action(
        detail=True,
        methods=['GET'],
    )
    def grupos(self, request, id):
        sucursalUserGroups = SucursalUserGroup.objects.all().filter(sucursal__id=id, user__id=request.user.id)
        suc = []
        for sucursalUserGroup in sucursalUserGroups:
            suc.append({'id': sucursalUserGroup.group.id, 'name': sucursalUserGroup.group.name})

        return Response(suc)

    @action(
        detail=True,
        methods=['GET'],
    )
    def Url(self, request, id):
        sucursal = Sucursal.objects.get(id=id)
        entidade = Entidade.objects.get(id=sucursal.entidade.id)
        return Response(str(entidade.tipo_entidade.id)+'/'+str(sucursal.entidade.id)+'/'+str(sucursal.id))
    
    

    @action(
        detail=True,
        methods=['GET'],
    )
    def getCapasSite(self, request, id):
        ficheiros = Ficheiros.objects.filter(sucursal__id=id, funcionalidade='CapaSite')
        ficheiros = FicheiroSerializer(ficheiros, many=True)
        data = (ficheiros.data)
        return Response(data, status=status.HTTP_200_OK)
    

    @action(
        detail=True,
        methods=['POST'],
    )
    def postCapasSite(self, request, id):
        sucursal = Sucursal.objects.get(id=id)

        request.data['entidade'] = str(sucursal.entidade.id)
        request.data['sucursal'] = str(sucursal.id)
        uploaded_file = request.FILES['ficheiro']

        if DiskManegarService.freeSpace(sucursal.entidade.id, request.FILES['ficheiro']):
            resposta = {'alert_error': 'Nao e possivel fazer upload de ficheiro<br><b>Contacte o adminstrador</b>'}
            return Response(resposta , status=status.HTTP_400_BAD_REQUEST)

        request.data['size'] = uploaded_file.size
        request.data['modelo'] = 'sucursal'
        request.data['estado'] = 'Activo'
        request.data['funcionalidade'] = 'CapaSite'

        ficheiro = FicheiroGravarSerializer(data=request.data)
        if ficheiro.is_valid(raise_exception=True):
            ficheiro.save()
            ficheiro = FicheiroSerializer(Ficheiros.objects.get(id=ficheiro.data['id']))
            DiskManegarService.updateSpace(sucursal.entidade.id, request.FILES['ficheiro'])
            return Response(ficheiro.data, status=status.HTTP_201_CREATED)
        else:
            return Response(ficheiro.errors, status=status.HTTP_400_BAD_REQUEST)
        
    

    @action(
        detail=True,
        methods=['GET'],
    )
    def qr(self, request, id):
        var_qr = {}
        origin = request.headers['Origin']
        LANGUAGE_CODE = 'pt-pt'

        TIME_ZONE = 'UTC'
        settings.LANGUAGE_CODE = 'pt-pt'
        # django.setup()
        print(settings.LANGUAGE_CODE)

        root = settings.MEDIA_ROOT
        lingua = self.request.query_params.get('lang')

        ean = barcode.get('code128', id, writer=ImageWriter())
        filename = ean.save(str(root) +'/' + str(random.random()) + 'qr' + str(random.random()))

        file = Image.open(str(filename))
        file = open(str(filename), 'rb').read()


        blob_barcode = base64.b64encode((file))
        if os.path.exists(filename):
            os.remove(filename)


        qr = qrcode.QRCode(box_size=2)
        qr.add_data(str('var_qr'))
        qr.make()
        img_qr = qr.make_image()
        # img_qr.
        img = img_qr.get_image()

        name = str(root) +'/' + str(random.random()) + 'qr' + str(random.random()) + '.png'
        img_qr.save(name)
        file = Image.open(str(name))
        file = open(str(name), 'rb').read()
        blob = base64.b64encode(bytes(file))
        if os.path.exists(name):
            os.remove(name)

        pk = id


        template_path = 'core/sucursal/qr_pdf.html'

        sucursal = Sucursal.objects.get(id=pk)
        sucursal1 = sucursal

    

        entidade = EntidadeSerializer(sucursal.entidade)

        sucursal = SucursalSerializer(sucursal)
        ficheiro  = Ficheiros.objects.get(entidade = sucursal1.entidade.id, funcionalidade = 'Logo')
        logo_name = ficheiro.ficheiro.path
        try:
            file = open(logo_name, 'rb').read()
            logo = base64.b64encode(bytes(file))
        except Exception as e:
            logo = logo_name.split('.')[-1]
        print(logo, logo_name, ficheiro)
        
        url = origin + '/#/?s=' + sucursal.data['id'] + '&q=1' 
        var_qr['entidade'] = entidade.data['nome']
        var_qr['sucursal'] = sucursal.data['nome']
        for key, value in var_qr.items():
            url = url + '&' + key + '=' + value
        # print(url)
        qr = qrcode.QRCode(box_size=2)
        qr.add_data(str(url))
        qr.make()
        img_qr = qr.make_image()
    

        name = str(root) +'/' + str(random.random()) + 'qr' + str(random.random()) + '.png'
        img_qr.save(name)
        file = Image.open(str(name))
        file = open(str(name), 'rb').read()
        qr_to_scan = base64.b64encode(bytes(file))


        if os.path.exists(name):
            os.remove(name)
        context = {
            'qr': blob,
            'qr_to_scan': qr_to_scan,
            'barcode': blob_barcode, 
            'entidade': entidade.data,
            'sucursal': sucursal.data,
            'logo':logo,
            'titulo': Translate.tdc(lingua, 'QR'),
            'nome': Translate.tdc(lingua, 'Sucursal'),
            'de': Translate.tdc(lingua, 'de'),
            'morada': Translate.tdc(lingua, 'Morada'),
            'pagina': Translate.tdc(lingua, 'Pagina')
        }

        return Response(context, status=status.HTTP_200_OK)





import base64
from io import BytesIO
from django.conf import settings
import os

import qrcode
import barcode
from barcode.writer import ImageWriter

from django.http import HttpResponse
from django.template.loader import render_to_string
from weasyprint import HTML

def png_bytes_to_b64(png_bytes: bytes) -> str:
    return base64.b64encode(png_bytes).decode("utf-8")


def make_qr_b64(text: str) -> str:
    img = qrcode.make(text)
    buf = BytesIO()
    img.save(buf, format="PNG")
    return png_bytes_to_b64(buf.getvalue())


def make_barcode_b64(value: str) -> str:
    CODE128 = barcode.get_barcode_class("code128")
    buf = BytesIO()
    CODE128(value, writer=ImageWriter()).write(buf, options={"quiet_zone": 2})
    return png_bytes_to_b64(buf.getvalue())

def PDF(template_path, request, doc=None, download=False, **context):
    doc = doc or {"name": "pdf"}

    if template_path.split('.')[-1] =='html':
        html = render_to_string(template_path, context)
    else:
        html = template_path



    pdf = HTML(string=html, base_url=request.build_absolute_uri("/")).write_pdf()

    filename = doc.get("name", "pdf").replace(" ", "_")

    response = HttpResponse(pdf, content_type="application/pdf")

    disposition = "attachment" if download else "inline"
    response["Content-Disposition"] = f'{disposition}; filename="{filename}.pdf"'

    return response





def get_cors_origins():
    try:
        from django_resaas.models.cors_allowed_origin import CorsAllowedOrigin
        return list(CorsAllowedOrigin.objects.values_list("origin", flat=True))
    except Exception:
        return []


import uuid

from django.db import models
from django.contrib.contenttypes.models import ContentType

from django_resaas.models.user import User
from django_resaas.models.entidade import Entidade
from django_resaas.core.base.models import TimeModel

class EntidadeUser(TimeModel):

    entidade = models.ForeignKey(Entidade, on_delete=models.CASCADE)
    user = models.ForeignKey(User, on_delete=models.CASCADE)

    class Meta:
        permissions = ()

    def __str__(self):
        return f'{self.entidade.nome} | {self.user.username}'